<?
/*

\		ignore next char
^\_		ignore this paragraph
^\$		ignore all til next occurence (or the end)
			
\\		\
_		&nbsp;
&		&amp;
"		&quot;

^+		<br />
^---$	<hr />

^#			<ol>
^-, ^*, ^.	<ul>
^::::_		h1 (_ is space)
^:::_		h2
^::_		h3
^:_			h4
^"_			blockquote
^{--$		<pre>
^--}$		</pre>

else ^.*$	<p>...</p>

*strong*
~em~
[URL]			<a href="URL">url</a>
[URL anything]	<a href="URL">anything</a>

||		comment line
|||		invisible line

empty line	ignore (until if pre)

{{...}}		code

{{?aaa=bbb}}		abbr (title bbb)
{{?aaa}}			abbr
{{"aaa=bbb}}		q
{{'aaa=bbb}}		cite


*/


function BXTagsParser(&$lines) {
	global $use_autoformat;

	$isXHTML = (strtolower($use_autoformat)=='xhtml');

	$esc = '\\';
	$brTag = ($isXHTML) ? 'br /':'br';
	$hrTag = ($isXHTML) ? 'hr /':'hr';

	$buff = '';
	$parse = true;
	
	$isPRE = false;
	$prevTag = '';
	$par = '';
	$prevl = '';
	$lCnt = count($lines);
	
	for ($i=0;$i<$lCnt;$i++) {

		$l = ereg_replace("\t"," ",chop($lines[$i]));

		// ESC starts/stops parsing
		if (!$parse) {
			if ($l==$esc) $parse = true;
			else $buff .= "$l\n";
			continue;
			}
		elseif ($l==$esc) {
			$parse = false;
			continue;
			}

		// ignore empty lines (but in PRE)
		if (!$isPRE && !trim($l)) continue;

		// + makes BR and joins with previous paragraph
		$l = ereg_replace("^\+(.*)","\\1",$l);
		if ($i<($lCnt-1) && substr($lines[$i+1],0,1)=='+') {
			$prevl .= $l."<$brTag>\n";
			continue;
			}
		else {
			$l = $prevl.$l;
			$prevl = '';
			}

		$stag = '';
		$etag = '';
		$comment = false;

		// % starts parsing if stopped
		if (ereg("^([^ ]+) +(.+)$",$l,$r)) {
			$fst = $r[1];
			$rest = $r[2];
			}
		else {
			$fst='';
			$rest='';
			}
		
	// special paragraphs
		// {{ starts PRE
		if ($l=='{--') {
			$par = '';
			$stag = 'pre';
			}
		// }} stops PRE
		elseif ($l=='--}') {
			$par = '';
			$etag = 'pre';
			}
		// --- is HR
		elseif (ereg("---+",$l) && !$rest) {
			$par = '';
			$stag = $hrTag;
			}
	// paragraph prefixes
		// ESC stops parsing line
		elseif ($fst==$esc) {
			$par = $rest;
			}
		// || comment line
		elseif ($fst=='||') {
			$par = $rest;
			$comment = true;
			}
		// ||| invisible line
		elseif ($fst=='|||') {
			$par = '';
			}
		// : - :::: makes H4-H1
		elseif (ereg("^(:+)",$fst,$r)) {
			if ($r[1]=='::::') $stag = 'h1';
			elseif ($r[1]==':::') $stag = 'h2';
			elseif ($r[1]=='::') $stag = 'h3';
			else $stag = 'h4';
			$etag = $stag;
			$par = parseLine($rest);
			}
		elseif ($fst=='"') {
			$stag = 'blockquote';
			$etag = 'blockquote';
			$par = parseLine($rest);
			}
		elseif ($fst=='#') {
			$stag = 'ol';
			$par = parseLine($rest);
			}
		elseif ($fst=='.' || $fst=='-' || $fst=='*') {
			$stag = 'ul';
			$par = parseLine($rest);
			}
		else {
			$par = parseLine($l);
			}

		$prefix = '';
		$par = chop($par);

		if ($stag=='pre') $isPRE = true;
		if (!$stag && !$isPRE && $par) { $stag='p'; $etag='p'; }

		if ($stag && $stag!='ol' && $prevTag=='ol') $prefix .= "</ol>\n";
		if ($stag && $stag!='ul' && $prevTag=='ul') $prefix .= "</ul>\n";
		if ($stag=='ol' && $prevTag!='ol') $prefix .= "<ol>\n";
		if ($stag=='ul' && $prevTag!='ul') $prefix .= "<ul>\n";

		if ($stag || $etag) $prevTag = ($stag) ? $stag : $etag;

		$t1 = ''; $t2 = '';
		if ($stag=='ol' || $stag=='ul') { $t1='   <li>'; $t2='</li>'; }
		else {
			if ($stag) $t1 = "<$stag>";
			if ($etag) $t2 = "</$etag>";
			}
		if ($comment) $par = "<!--$par-->";
		else $par = $prefix.$t1.$par.$t2;
		if ($par || $isPRE) $buff .=  $par . "\n";

		if ($etag=='pre') $isPRE = false;
		}

	if ($prevTag=='ol') $buff .= "</ol>\n";
	if ($prevTag=='ul') $buff .= "</ul>\n";

	return $buff;
	}


function parseLine($txt) {
	$esc = '\\';
	
	$buff = '';

	$HTML = false;
	
	$isStrong = false;
	$isEm = false;
	$isTagStart = false;
	$isTagEnd = false;

	$tagCnt = -1;

	$txt = preg_replace("/(\\$esc(.))/e","'#esc#'.ord('\\2').'#esc#'",$txt);

	$txt = ereg_replace('\[\?([^]]*)\|([^]]*)\]',"<abbr title='\\2'>\\1</abbr>",$txt);
	$txt = ereg_replace('\[\?([^]]*)\]',"<abbr>\\1</abbr>",$txt);
	$txt = ereg_replace("\[([^]]*)\|([^]]*)\]","<a href='\\1'>\\2</a>",$txt);
	$txt = ereg_replace("\[([^]]*)]","<a href='\\1'>\\1</a>",$txt);

	$sl = strlen($txt);
	for ($i=0;$i<$sl;$i++) {
		$a = substr($txt,$i,1);
		$rest = substr($txt,$i+1);

		if ($a=='<') $HTML = true;
		if ($HTML) {
			$buff .= $a;
			if ($a=='>') $HTML = false;
			continue;
			}
		
		if ($a=='*') {
			$buff .= (!$isStrong) ? '<strong>':'</strong>';
			$isStrong = !$isStrong;
			}
		elseif ($a=='~') {
			$buff .= (!$isEm) ? '<em>':'</em>';
			$isEm = !$isEm;
			}
		elseif ($a=='{') {
			if ($prev=='{') $tagStart = true;
			}
		elseif ($a=='}') {
			if ($prev=='}') {
				if ($tagCnt>=0) {
					$buff .= "</".$tag[$tagCnt].">";
					$tagCnt--;
					$a = '';
					}
				}
			}
		else {
			if ($tagStart) {
				$tagCnt++;
				if ($a=='"') {
					$tag[$tagCnt] = 'q';
					$a = '';
					}
				elseif ($a=="'") {
					$tag[$tagCnt] = 'cite';
					$a = '';
					}
				else $tag[$tagCnt] = 'code';
				$buff .= '<'.$tag[$tagCnt].'>'.$a;
				}
			elseif ($a=='_') {
				$buff .= '&nbsp;';
				}
			elseif ($a=='"') {
				$buff .= '&quot;';
				}
			else {
				if ($prev=='{') $buff .= '{';
				elseif ($prev=='}') $buff .= '}';
				$buff .= $a;
				}
			$tagStart = false;
			}
		$prev = $a;
		}

	$buff = preg_replace("/&(?!\#?[a-z0-9]+;)/","&amp;",$buff);
	$buff = preg_replace("/#esc#60#esc#/","&lt;",$buff);
	$buff = preg_replace("/#esc#62#esc#/","&gt;",$buff);
	$buff = preg_replace("/#esc#([0-9]+)#esc#/e","chr(\\1)",$buff);

	return $buff;
	
	}

?>